// ------------------------------------------------------------------
// CLOAKING PACK
//
// When activated, this pack cloaks the user from all visual detection
// by other players and cameras. This is a local effect only (someone standing
// next to the cloaked player will still be visible). Motion sensors will
// still detect a (moving) cloaked player.
//
// When not activated, the pack creates a localized passive sensor dampening
// field.  The user will not be visible to any non-motion-detecting sensor.
//

datablock EffectProfile(CloakingPackActivateEffect)
{
   effectname = "packs/cloak_on";
   minDistance = 2.5;
   maxDistance = 2.5;
};

datablock AudioProfile(CloakingPackActivateSound)
{
   filename = "fx/packs/cloak_on.wav";
   description = CloseLooping3d;
   preload = true;
   effect = CloakingPackActivateEffect;
};

datablock ShapeBaseImageData(CloakingPackImage)
{
   shapeFile = "pack_upgrade_cloaking.dts";
   item = CloakingPack;
   mountPoint = 1;
   offset = "0 0 0";

   usesEnergy = true;
   minEnergy = 1.5;    //was 3 -Phantom139

   stateName[0] = "Idle";
   stateTransitionOnTriggerDown[0] = "Activate";
   
   stateName[1] = "Activate";
   stateScript[1] = "onActivate";
   stateSequence[1] = "fire";
   stateSound[1] = CloakingPackActivateSound;
   stateEnergyDrain[1] = 12;
   stateTransitionOnTriggerUp[1] = "Deactivate";
   stateTransitionOnNoAmmo[1] = "Deactivate";

   stateName[2] = "Deactivate";
   stateScript[2] = "onDeactivate";
   stateTransitionOnTimeout[2] = "Idle";
};

datablock ItemData(CloakingPack)
{
   className = Pack;
   catagory = "Packs";
   shapeFile = "pack_upgrade_cloaking.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   pickupRadius = 2;
   rotate = true;
   image = "CloakingPackImage";
   pickUpName = "a cloaking pack";

   computeCRC = true;

};

function CloakingPackImage::onMount(%data, %obj, %node)
{
   // player is sensor-invisible while wearing pack (except to motion sensors)
   %obj.setPassiveJammed(true);
}

function CloakingPackImage::onUnmount(%data, %obj, %node)
{
   %obj.setPassiveJammed(false);
   %obj.setCloaked(false);
   %obj.setImageTrigger(%node, false);
}

// make player completely invisible to all players/sensors (except motion)
function CloakingPackImage::onActivate(%data, %obj, %slot)
{
   if(%obj.reCloak !$= "")
   {   
      Cancel(%obj.reCloak);
      %obj.reCloak = "";
   }
      // can the player currently cloak (function returns "true" or reason for failure)?
      if(%obj.canCloak() $= "true")
      {
         messageClient(%obj.client, 'MsgCloakingPackOn', '\c2Cloaking pack on.');
         %obj.setCloaked(true);
         if ( !isDemo() )
            commandToClient( %obj.client, 'setCloakIconOn' );
      }
      else
      {
         // notify player that they cannot cloak
         messageClient(%obj.client, 'MsgCloakingPackFailed', '\c2Jamming field prevents cloaking.');
         %obj.setImageTrigger(%slot, false);
      }
}

function CloakingPackImage::onDeactivate(%data, %obj, %slot)
{
   if(%obj.reCloak !$= "")
   {   
      Cancel(%obj.reCloak);
      %obj.reCloak = "";
   }
   
   // if pack is not on then dont bother...
   if(%obj.getImageState($BackpackSlot) $= "activate")
      messageClient(%obj.client, 'MsgCloakingPackOff', '\c2Cloaking pack off.');

   %obj.setCloaked(false);
   %obj.setImageTrigger(%slot, false);
   if ( !isDemo() )
      commandToClient( %obj.client, 'setCloakIconOff' );
}

function CloakingPack::onPickup(%this, %obj, %shape, %amount)
{
   // created to prevent console errors
}

function ShapeBaseData::onForceUncloak(%this, %obj, %reason)
{
   // dummy
}

function Armor::onForceUncloak(%this, %obj, %reason)
{
   %pack = %obj.getMountedImage($BackpackSlot);
   if((%pack <= 0) || (%pack.item !$= "CloakingPack"))
      return;
   
   if(%obj.getImageState($BackpackSlot) $= "activate")
   {
      // cancel recloak thread
      if(%obj.reCloak !$= "")
      {   
         Cancel(%obj.reCloak);
         %obj.reCloak = "";
      }

      messageClient(%obj.client, 'MsgCloakingPackOff', '\c2Cloaking pack off.  Jammed.');
      %obj.setCloaked(false);
      %obj.setImageTrigger($BackpackSlot, false);
   }
}
//****************************************************************************************
// function CloakLoop- code by -BadShot
//****************************************************************************************

function CloakLoop()
{
   %packradius = 5; //Radius of cloaking pack mass cloak
   if (!(isObject(MissionCleanup)))
   {
      schedule(10000, 0, "StartCloakLoop");
      return;
   }
   %dep = nameToID("MissionCleanup/Deployables");
   %count = ClientGroup.getCount();
   %count2 = %dep.getCount();
   %count3 = MissionCleanup.getCount();
   for (%i=0;%i<%count;%i++)
   {
      ClientGroup.getObject(%i).shouldCloak = 0;
   }
   for (%i=0;%i<%count2;%i++)
   {
      %dep.getObject(%i).shouldCloak = 0;
   }
   for (%i=0;%i<%count3;%i++)
   {
      %obj = MissionCleanup.getObject(%i);
      if (%obj.getType() & $TypeMasks::VehicleObjectType)
         %obj.shouldCloak = 0;
   }
   for (%i=0;%i<%count;%i++)
   {
      %obj = ClientGroup.getObject(%i).player;
      if(%obj)
      {
         if ((%obj.getMountedImage($BackpackSlot) $= CloakingPackImage.getID()) && (%obj.getImageState($BackpackSlot) $= "activate"))
         {
            %obj.client.shouldCloak = 1;
            %pos = %obj.getPosition();
            for(%i2=0;%i2<%count;%i2++) //Pack cloaks players
            {
               %obj2 = ClientGroup.getObject(%i2).player;
               if ((%obj2) && (%obj2 != %obj) && (%obj2.team == %obj.team))
               {
                  %pos2 = %obj2.getPosition();
                  if (VectorDist(%pos, %pos2) < %packradius)
                     %obj2.client.shouldCloak = 1;
               }
            }
            for(%i2=0;%i2<%count2;%i2++) //Pack cloaks deployables
            {
               %obj2 = %dep.getObject(%i2);
               if ((%obj2) && (%obj2 != %obj) && (%obj2.team == %obj.team))
               {
                  %pos2 = posFromTransform(%obj2.getTransform());
                  if (VectorDist(%pos, %pos2) < %packradius)
                     %obj2.shouldCloak = 1;
               }
            }
            for (%i2=0;%i2<%count3;%i2++) //Pack cloaks vehicles
            {
               %obj2 = MissionCleanup.getObject(%i2);
               if ((%obj2.getType() & $TypeMasks::VehicleObjectType) && (%obj2.team == %obj.team))
               {
                  %pos2 = posFromTransform(%obj2.getTransform());
                  if (VectorDist(%pos, %pos2) < %packradius)
                     %obj2.shouldCloak = 1;
               }
            }
         }
      }
   }
   for (%i=0;%i<%count2;%i++)
   {
      %obj = %dep.getObject(%i);
      if (isObject(%obj))
      {
         %data = %obj.getDataBlock();
         if ((%data.cloakOthers) && (%obj.getDamageState() $= "Enabled"))
         {
            %pos = posFromTransform(%obj.getTransform());
            for(%i2=0;%i2<%count;%i2++) //Deployables cloak players
            {
               %obj2 = ClientGroup.getObject(%i2).player;
               if ((%obj2) && (%obj2 != %obj) && (%obj2.team == %obj.team))
               {
                  %pos2 = %obj2.getPosition();
                  if (VectorDist(%pos, %pos2) < %data.cloakRadius)
                     %obj2.client.shouldCloak = 1;
               }
            }
            for(%i2=0;%i2<%count2;%i2++) //Deployables cloak deployables
            {
               %obj2 = %dep.getObject(%i2);
               if ((%obj2) && (%obj2 != %obj) && (%obj2.team == %obj.team))
               {
                  %pos2 = posFromTransform(%obj2.getTransform());
                  if (VectorDist(%pos, %pos2) < %data.cloakRadius)
                     %obj2.shouldCloak = 1;
               }
            }
            for (%i2=0;%i2<%count3;%i2++) //Deployables cloak vehicles
            {
               %obj2 = MissionCleanup.getObject(%i2);
               if ((%obj2.getType() & $TypeMasks::VehicleObjectType) && (%obj2.team == %obj.team))
               {
                  %pos2 = posFromTransform(%obj2.getTransform());
                  if (VectorDist(%pos, %pos2) < %data.cloakRadius)
                     %obj2.shouldCloak = 1;
               }
            }
         }
      }
   }
   for (%i=0;%i<%count3;%i++)
   {
      %obj = MissionCleanup.getObject(%i);
      if (%obj.getType() & $TypeMasks::SensorObjectType)
      {
         %data = %obj.getDataBlock();
         if ((%data.cloakOthers) && (%obj.getDamageState() $= "Enabled"))
         {
            %pos = posFromTransform(%obj.getTransform());
            for(%i2=0;%i2<%count;%i2++) //Beacons cloak players
            {
               %obj2 = ClientGroup.getObject(%i2).player;
               if ((%obj2) && (%obj2 != %obj) && (%obj2.team == %obj.team))
               {
                  %pos2 = %obj2.getPosition();
                  if (VectorDist(%pos, %pos2) < %data.cloakRadius)
                     %obj2.client.shouldCloak = 1;
               }
            }
            for(%i2=0;%i2<%count2;%i2++) //Beacons cloak deployables
            {
               %obj2 = %dep.getObject(%i2);
               if ((%obj2) && (%obj2 != %obj) && (%obj2.team == %obj.team))
               {
                  %pos2 = posFromTransform(%obj2.getTransform());
                  if (VectorDist(%pos, %pos2) < %data.cloakRadius)
                     %obj2.shouldCloak = 1;
               }
            }
            for (%i2=0;%i2<%count3;%i2++) //Beacons cloak vehicles
            {
               %obj2 = MissionCleanup.getObject(%i2);
               if ((%obj2.getType() & $TypeMasks::VehicleObjectType) && (%obj2.team == %obj.team))
               {
                  %pos2 = posFromTransform(%obj2.getTransform());
                  if (VectorDist(%pos, %pos2) < %data.cloakRadius)
                     %obj2.shouldCloak = 1;
               }
            }
         }
      }
   }
   for (%i=0;%i<%count;%i++) //Cloaks players
   {
      %client = ClientGroup.getObject(%i);
      %obj = %client.player;
      if (%obj)
      {
         if ((%client.shouldCloak) && (!%obj.isCloaked()))
            %obj.setCloaked(true);
         else if ((!%client.shouldCloak) && (%obj.isCloaked()))
            %obj.setCloaked(false);
      }
   }
   for (%i=0;%i<%count2;%i++) //Cloaks deployables
   {
      %obj = %dep.getObject(%i);
      if (isObject(%obj))
      {
         %data = %obj.getDataBlock();
         if (%data.cloakOthers)
         {
            if (%obj.isCloaked())
               %obj.setCloaked(false);
         }
         else if ((%obj.shouldCloak) && (!%obj.isCloaked()))
            %obj.setCloaked(true);
         else if ((!%obj.shouldCloak) && (%obj.isCloaked()))
            %obj.setCloaked(false);
      }
   }
   for (%i=0;%i<%count3;%i++) //Cloaks vehicles
   {
      %obj = MissionCleanup.getObject(%i);
      if (%obj.getType() & $TypeMasks::VehicleObjectType)
      {
         if ((%obj.shouldCloak) && (!%obj.isCloaked()))
            %obj.setCloaked(true);
         else if ((!%obj.shouldCloak) && (%obj.isCloaked()))
            %obj.setCloaked(false);
      }
   }

   schedule(1000, 0, "cloakLoop");
}

function StartCloakLoop()
{
   $CloakLoop = 1;
   if (!(isObject(MissionCleanup)))
   {
      schedule(10000, 0, "StartCloakLoop");
      return;
   }
   %depGroup = nameToID("MissionCleanup/Deployables");
   if(%depGroup <= 0) {
      %depGroup = new SimGroup("Deployables");
      MissionCleanup.add(%depGroup);
   }
   CloakLoop();
}
